/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.asset.publisher.web.configuration;

import aQute.bnd.annotation.metatype.Meta;

import com.liferay.portal.configuration.metatype.annotations.ExtendedObjectClassDefinition;

/**
 * @author Pavel Savinov
 */
@ExtendedObjectClassDefinition(category = "assets")
@Meta.OCD(
	id = "com.liferay.asset.publisher.web.configuration.AssetPublisherWebConfiguration",
	localization = "content/Language",
	name = "asset-publisher-web-configuration-name"
)
public interface AssetPublisherWebConfiguration {

	/**
	 * Set the interval in hours on how often CheckAssetEntryMessageListener
	 * will run to check for new assets. Users will be notified via email of new
	 * assets.
	 *
	 * @return interval in hours on how often to check for new assets.
	 */
	@Meta.AD(
		deflt = "24", description = "check-interval-key-description",
		name = "check-interval", required = false
	)
	public int checkInterval();

	/**
	 * Set this to <code>true</code> to enable exporting contents related to
	 * asset entries for dynamic selection.
	 *
	 * @return <code>true</code> if dynamic export is enabled.
	 */
	@Meta.AD(
		deflt = "true", description = "dynamic-export-enabled-key-description",
		name = "dynamic-export-enabled", required = false
	)
	public boolean dynamicExportEnabled();

	/**
	 * Set the maximum number of entries to export for dynamic selection. Set
	 * this property to 0 if there is no limit.
	 *
	 * @return maximum number of entries to export for dynamic selection.
	 */
	@Meta.AD(
		deflt = "20", description = "dynamic-export-limit-key-description",
		name = "dynamic-export-limit", required = false
	)
	public int dynamicExportLimit();

	/**
	 * Set the maximum number of new entries to notify subscribers of Asset
	 * Publisher portlets. Set this property to 0 if there is no limit.
	 *
	 * @return maximum number of new entries to notify subscribers.
	 */
	@Meta.AD(
		deflt = "20",
		description = "dynamic-subscription-limit-key-description",
		name = "dynamic-subscription-limit", required = false
	)
	public int dynamicSubscriptionLimit();

	/**
	 * Set this to <code>true</code> to enable exporting contents related to
	 * asset entries for manual selection.
	 *
	 * @return <code>true</code> if manual export is enabled.
	 */
	@Meta.AD(
		deflt = "true", description = "manual-export-enabled-key-description",
		name = "manual-export-enabled", required = false
	)
	public boolean manualExportEnabled();

	/**
	 * Set this to <code>true</code> to search assets in Asset Publisher from
	 * the index. Set this to <code>false</code> to search assets in Asset
	 * Publisher from the database.
	 *
	 * @return <code>true</code> search with index is enabled.
	 */
	@Meta.AD(
		deflt = "true", description = "search-with-index-key-description",
		name = "search-with-index", required = false
	)
	public boolean searchWithIndex();

	/**
	 * Set this to <code>true</code> to allow users to configure Asset
	 * Publisher, Most Viewed Assets, and Highest Rated Assets to skip the
	 * permissions checking on the displayed assets. Enabling this property will
	 * allow regular users to view assets that they do not have permission to
	 * view.
	 *
	 * @return <code>true</code> if permission checking is configurable.
	 */
	@Meta.AD(
		deflt = "false",
		description = "permission-checking-configurable-key-description",
		name = "permission-checking-configurable", required = false
	)
	public boolean permissionCheckingConfigurable();

}